const { makeExecutableSchema } = require("@graphql-tools/schema");

const UserModel = require("../models/User");
const axios = require('axios');

const typeDefs = `
    type User {
        _id: ID!
        username: String!
        password: String
        isAgent: Boolean
    }
    
    input UserInput {
        username: String!
        password: String
        isAgent: Boolean
    }
    
    type AuthPayload {
        token: String!
    }
    
    type Query {
        getUser(id: ID, username: String): User
    }
    
    type Mutation {
        createUser(input: UserInput): User
        loginUser(username: String!, password: String!): AuthPayload
        logoutUser: String
    }
`;

const resolvers = {
    Query: {
        getUser: async (parent, { id, username }) => {
            if (id) {
                return await UserModel.findById(id);
            } else if (username) {
                return await UserModel.findOne({ username });
            } else {
                return null;
            }
        }
    },
    Mutation: {
        createUser: async (parent, { input }) => {
            try {
                const response = await axios.post('http://localhost:8080/user', input);
                if (response.data.exports){
                    throw new Error(response.data.exports.error);
                }
                return response.data.user;
            } catch (error) {
                console.error('Erreur lors de la création de l\'utilisateur:', error);
                throw new Error('Erreur lors de la création de l\'utilisateur. Vérifiez les détails dans les journaux.');
            }
        },
        loginUser: async (parent, { username, password }) => {
            try {
                const response = await axios.post('http://localhost:8080/user/login', { username, password });

                if (response.status === 200) {
                    return { token: response.data.token };
                } else {
                    throw new Error('Invalid username or password.');
                }
            } catch (error) {
                console.error('Erreur lors de la tentative de connexion:', error);
                throw new Error('Erreur lors de la tentative de connexion. Vérifiez les détails dans les journaux.');
            }
        },
        logoutUser: async () => {
            try {
                const response = await axios.post('http://localhost:8080/user/logout');

                if (response.status === 200) {
                    return 'Logout successful.';
                } else {
                    throw new Error('Logout failed.');
                }
            } catch (error) {
                console.error('Erreur lors de la déconnexion de l\'utilisateur:', error);
                throw new Error('Erreur lors de la déconnexion de l\'utilisateur. Vérifiez les détails dans les journaux.');
            }
        },
    },
};

exports.schema = makeExecutableSchema({ typeDefs, resolvers });