const Ad = require('../models/Ad');
const User = require('../models/User');

const {extractUserFromToken} = require("./UserService");

const secretKey = 'secretKey';

exports.addAd = function (body, token) {
    return new Promise(async (resolve, reject) => {

        try {
            const { title, propertyType, publicationStatus, propertyStatus, description, price, availabilityDate} = body;
            const photos = [];
            /*
            req.files.forEach(file => {
                photos.push(file.filename);
            });
            */
            const decoded = await extractUserFromToken(token, secretKey);
            const user = await User.findOne({ username: decoded.username });

            if (user.isAgent === false) {
                return resolve({ status: 403, message: 'User non autorisée.' });
            }
            const newAnnouncement = new Ad({
                title,
                propertyType,
                publicationStatus,
                propertyStatus,
                description,
                price,
                availabilityDate,
                photos,
                userName: user.username
            });

            await newAnnouncement.save();
            //res.status(200).json(newAnnouncement);
            resolve({ status: 200, message: 'Annonce Crée avec succes' });
        } catch (error) {
            reject(error);
        }
    });
}

// Méthode pour mettre à jour une annonce
exports.updateAd = function (announcementId, reqBody, token) {
  return new Promise(async (resolve, reject) => {
    try {
        const user = await extractUserFromToken(token, secretKey);
      const { title, propertyType, publicationStatus, propertyStatus, description, price, availabilityDate } = reqBody;
      /*
      let photos = [];
      const oldPhotos = reqBody.oldPhotos || [];

      reqFiles.forEach(file => {
        photos.push(file.filename);
      });

      photos = photos.concat(oldPhotos);
      */
      const updatedAnnouncement = await Ad.findByIdAndUpdate(
          announcementId,
          {
            title,
            propertyType,
            publicationStatus,
            propertyStatus,
            description,
            price,
            availabilityDate,
            //photos,
          }
      );
      if (!updatedAnnouncement) {
          return resolve({status: 404, message: 'annonce non trouvée'});
      }
      if (user.username !== updatedAnnouncement.userName) {
          return resolve({status: 401, message: 'Utilisateur non autorisé'});
      }

      resolve({ status: 200, message: 'Annonce Modifié avec succès' });
    } catch (error) {
      reject(error);
    }
  });
}

exports.deleteAd = function (announcementId, token) {
  return new Promise(async (resolve, reject) => {
    try {
        const user = await extractUserFromToken(token, secretKey);
        const Announcement = await Ad.findById(announcementId);
        if (user.username !== Announcement.userName) {
            return resolve({ status: 403, message: 'User non autorisé' });
        }

      const deletedAnnouncement = await Ad.findByIdAndRemove(announcementId);

      if (!deletedAnnouncement) {
        return resolve({ status: 404, message: 'annonce non trouvée' });
      }

      /*
      deletedAnnouncement.photos.forEach((photo) => {
        const filePath = path.join('public', 'uploads', photo);
        if (fs.existsSync(filePath)) {
          fs.unlinkSync(filePath);
        }
      });
      */
      resolve({ status: 200, message: 'Annonce supprimée avec succès' });
    } catch (error) {
      reject(error);
    }
  });
};

exports.getAds = function (token) {
  return new Promise(async (resolve, reject) => {
    try {
        const decoded = await extractUserFromToken(token, secretKey);
        let announcements;
        const user = await User.findOne({ username: decoded.username });
        if (user && user.isAgent) {
            announcements = await Ad.find({ userName: user.username });
        } else {
            announcements = await Ad.find({ publicationStatus: 'Publiée' });
        }
        resolve(announcements);
    } catch (error) {
      reject(error);
    }
  });
};

exports.getAdById = function (announcementId, token) {
  return new Promise(async (resolve, reject) => {
    try {
        const announcementDetails = await Ad.findById(announcementId);
        const decoded = await extractUserFromToken(token, secretKey);
        const user = await User.findOne({ username: decoded.username });


        if (
          (user && user.username !== announcementDetails.userName || !user) &&
          announcementDetails.publicationStatus !== 'Publiée'
          ) {
            return resolve({ status: 404, message: 'Cette annonce est introuvable ou non disponible.' });
          }
          if (user && user.username !== announcementDetails.userName && user.isAgent) {
            return resolve({ status: 403, message: 'User non autorisée.' });
          }

          if( !announcementDetails ) {
            return resolve({ status: 404, message: 'Cette annonce est introuvable ou non disponible.' });
          }
          resolve(announcementDetails);
    } catch (error) {
      reject(error);
    }
  });
};

// Service pour poser une question sur une annonce
exports.askQuestion = async function(announcementId, question, token) {
  return new Promise(async (resolve, reject) => {
      try {
      const announcement = await Ad.findById(announcementId);
      const decoded = await extractUserFromToken(token, secretKey);
      const user = await User.findOne({ username: decoded.username });

      if (!announcement) {
        return resolve({ status: 404, message: 'Annonce non trouvée.' });
      }
      if (announcement.propertyStatus !== 'Disponible') {
        return resolve({ status: 404, message: 'Vous ne pouvez poser une question que sur les annonces non disponibles.' });
      }
      announcement.questions.push({
        user: user.username,
        question
      });
      await announcement.save();
      resolve({ status: 200, message: 'Question posée avec succès.' });
    } catch (error) {
      throw error;
    }
  });
};

// Service pour répondre à une question sur une annonce
exports.answerQuestion = async function(announcementId, questionId, answer, token) {
  return new Promise(async (resolve, reject) => {
      try {
      const announcement = await Ad.findById(announcementId);
      const decoded = await extractUserFromToken(token, secretKey);
      const user = await User.findOne({ username: decoded.username });

      if (!announcement) {
        return resolve({ status: 404, message: 'Annonce non trouvée.' });
      }

      if (user.username !== announcement.userName) {
          return resolve({ status: 403, message: 'User non autorisée.' });
      }

      const question = announcement.questions.id(questionId);
      if (!question) {
        return resolve({ status: 404, message: 'Question non trouvée.' });
      }
      question.answers.push({ user: user.username, answer });
      await announcement.save();
      resolve({ status: 200, message: 'Réponse ajoutée avec succès.' });
    } catch (error) {
      throw error;
    }
  });
}
